@@ -48,7 +48,6 @@ on_name_vanished (GDBusConnection *connection,
 	g_print ("Name '%s' does not exist on the system bus => NM is not running\n", name);
 }
 
-
 int
 main (int argc, char *argv[])
 {
@@ -56,11 +55,6 @@ main (int argc, char *argv[])
 	GMainLoop *loop;
 	GBusNameWatcherFlags flags;
 
-#if !GLIB_CHECK_VERSION (2, 35, 0)
-	/* Initialize GType system */
-	g_type_init ();
-#endif
-
 	g_print ("Monitor 'org.freedesktop.NetworkManager' D-Bus name\n");
 	g_print ("===================================================\n");
 
