@@ -22,11 +22,18 @@
 #ifndef __NM_KEYFILE_INTERNAL_H__
 #define __NM_KEYFILE_INTERNAL_H__
 
+#if !((NETWORKMANAGER_COMPILATION) & NM_NETWORKMANAGER_COMPILATION_WITH_LIBNM_CORE_INTERNAL)
+#error Cannot use this header.
+#endif
+
 #include <sys/types.h>
 
 #include "nm-connection.h"
 #include "nm-setting-8021x.h"
 
+#include "nm-core-internal.h"
+#include "nm-meta-setting.h"
+
 /*****************************************************************************/
 
 #define NM_KEYFILE_CERT_SCHEME_PREFIX_PATH "file://"
@@ -87,7 +94,6 @@ typedef struct {
 	const char *message;
 } NMKeyfileReadTypeDataWarn;
 
-
 NMConnection *nm_keyfile_read (GKeyFile *keyfile,
                                const char *keyfile_name,
                                const char *base_dir,
@@ -138,20 +144,10 @@ typedef gboolean (*NMKeyfileWriteHandler) (NMConnection *connection,
  * type %NM_KEYFILE_WRITE_TYPE_CERT.
  */
 typedef struct {
+	const NMSetting8021xSchemeVtable *vtable;
 	NMSetting8021x *setting;
-	const char *property_name;
-
-	/* The following functions are helpers that simplify the implementation
-	 * of the handler. */
-	const char *suffix;
-	NMSetting8021xCKScheme (*scheme_func) (NMSetting8021x *setting);
-	NMSetting8021xCKFormat (*format_func) (NMSetting8021x *setting);
-	const char *           (*path_func)   (NMSetting8021x *setting);
-	GBytes *               (*blob_func)   (NMSetting8021x *setting);
-	const char *           (*uri_func)    (NMSetting8021x *setting);
 } NMKeyfileWriteTypeDataCert;
 
-
 GKeyFile *nm_keyfile_write (NMConnection *connection,
                             NMKeyfileWriteHandler handler,
                             void *user_data,
@@ -167,5 +163,4 @@ gboolean _nm_keyfile_a_contains_all_in_b (GKeyFile *kf_a, GKeyFile *kf_b);
 gboolean _nm_keyfile_equals (GKeyFile *kf_a, GKeyFile *kf_b, gboolean consider_order);
 gboolean _nm_keyfile_has_values (GKeyFile *keyfile);
 
-
 #endif /* __NM_KEYFILE_INTERNAL_H__ */
