@@ -51,7 +51,7 @@
  **/
 
 G_DEFINE_TYPE_WITH_CODE (NMSettingIP4Config, nm_setting_ip4_config, NM_TYPE_SETTING_IP_CONFIG,
-                         _nm_register_setting (IP4_CONFIG, 4))
+                         _nm_register_setting (IP4_CONFIG, NM_SETTING_PRIORITY_IP))
 NM_SETTING_REGISTER_TYPE (NM_TYPE_SETTING_IP4_CONFIG)
 
 #define NM_SETTING_IP4_CONFIG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), NM_TYPE_SETTING_IP4_CONFIG, NMSettingIP4ConfigPrivate))
@@ -191,7 +191,7 @@ verify (NMSetting *setting, NMConnection *connection, GError **error)
 		return FALSE;
 	}
 
-	if (priv->dhcp_client_id && !strlen (priv->dhcp_client_id)) {
+	if (priv->dhcp_client_id && !priv->dhcp_client_id[0]) {
 		g_set_error_literal (error,
 		                     NM_CONNECTION_ERROR,
 		                     NM_CONNECTION_ERROR_INVALID_PROPERTY,
@@ -363,7 +363,7 @@ ip4_addresses_set (NMSetting  *setting,
 	if (g_variant_lookup (s_ip4, "address-labels", "^as", &labels)) {
 		for (i = 0; i < addrs->len && labels[i]; i++)
 			if (*labels[i])
-				nm_ip_address_set_attribute (addrs->pdata[i], "label", g_variant_new_string (labels[i]));
+				nm_ip_address_set_attribute (addrs->pdata[i], NM_IP_ADDRESS_ATTRIBUTE_LABEL, g_variant_new_string (labels[i]));
 		g_strfreev (labels);
 	}
 	g_variant_unref (s_ip4);
@@ -391,7 +391,7 @@ ip4_address_labels_get (NMSetting    *setting,
 	num_addrs = nm_setting_ip_config_get_num_addresses (s_ip);
 	for (i = 0; i < num_addrs; i++) {
 		NMIPAddress *addr = nm_setting_ip_config_get_address (s_ip, i);
-		GVariant *label = nm_ip_address_get_attribute (addr, "label");
+		GVariant *label = nm_ip_address_get_attribute (addr, NM_IP_ADDRESS_ATTRIBUTE_LABEL);
 
 		if (label) {
 			have_labels = TRUE;
@@ -404,7 +404,7 @@ ip4_address_labels_get (NMSetting    *setting,
 	labels = g_ptr_array_sized_new (num_addrs);
 	for (i = 0; i < num_addrs; i++) {
 		NMIPAddress *addr = nm_setting_ip_config_get_address (s_ip, i);
-		GVariant *label = nm_ip_address_get_attribute (addr, "label");
+		GVariant *label = nm_ip_address_get_attribute (addr, NM_IP_ADDRESS_ATTRIBUTE_LABEL);
 
 		g_ptr_array_add (labels, (char *) (label ? g_variant_get_string (label, NULL) : ""));
 	}
@@ -524,7 +524,6 @@ ip4_route_data_set (NMSetting  *setting,
 	return TRUE;
 }
 
-
 static void
 nm_setting_ip4_config_class_init (NMSettingIP4ConfigClass *ip4_class)
 {
@@ -615,7 +614,7 @@ nm_setting_ip4_config_class_init (NMSettingIP4ConfigClass *ip4_class)
 	 * ---end---
 	 * ---ifcfg-rh---
 	 * property: routes
-	 * variable: ADDRESS1, NETMASK1, GATEWAY1, METRIC1, ...
+	 * variable: ADDRESS1, NETMASK1, GATEWAY1, METRIC1, OPTIONS1, ...
 	 * description: List of static routes. They are not stored in ifcfg-* file,
 	 *   but in route-* file instead.
 	 * ---end---
@@ -680,6 +679,22 @@ nm_setting_ip4_config_class_init (NMSettingIP4ConfigClass *ip4_class)
 	 * ---end---
 	 */
 
+	/* ---ifcfg-rh---
+	 * property: route-table
+	 * variable: IPV4_ROUTE_TABLE(+)
+	 * default: 0
+	 * description: IPV4_ROUTE_TABLE enables policy-routing and sets the default routing table.
+	 * ---end---
+	 */
+
+	/* ---ifcfg-rh---
+	 * property: dns-options
+	 * variable: RES_OPTIONS(+)
+	 * description: List of DNS options to be added to /etc/resolv.conf
+	 * example: RES_OPTIONS=ndots:2 timeout:3
+	 * ---end---
+	 */
+
 	/* ---ifcfg-rh---
 	 * property: dns-priority
 	 * variable: IPV4_DNS_PRIORITY(+)
@@ -696,12 +711,31 @@ nm_setting_ip4_config_class_init (NMSettingIP4ConfigClass *ip4_class)
 	 *
 	 * A string sent to the DHCP server to identify the local machine which the
 	 * DHCP server may use to customize the DHCP lease and options.
+	 * When the property is a hex string ('aa:bb:cc') it is interpreted as a
+	 * binary client ID, in which case the first byte is assumed to be the
+	 * 'type' field as per RFC 2132 section 9.14 and the remaining bytes may be
+	 * an hardware address (e.g. '01:xx:xx:xx:xx:xx:xx' where 1 is the Ethernet
+	 * ARP type and the rest is a MAC address).
+	 * If the property is not a hex string it is considered as a
+	 * non-hardware-address client ID and the 'type' field is set to 0.
+	 *
+	 * The special values "mac" and "perm-mac" are supported, which use the
+	 * current or permanent MAC address of the device to generate a client identifier
+	 * with type ethernet (01). Currently, these options only work for ethernet
+	 * type of links.
+	 *
+	 * The special value "stable" is supported to generate a type 0 client identifier based
+	 * on the stable-id (see connection.stable-id) and a per-host key.
+	 *
+	 * If unset, a globally configured default is used. If still unset, the
+	 * client-id from the last lease is reused.
 	 **/
 	/* ---ifcfg-rh---
 	 * property: dhcp-client-id
 	 * variable: DHCP_CLIENT_ID(+)
 	 * description: A string sent to the DHCP server to identify the local machine.
-	 * example: DHCP_CLIENT_ID=ax-srv-1
+	 *    A binary value can be specified using hex notation ('aa:bb:cc').
+	 * example: DHCP_CLIENT_ID=ax-srv-1; DHCP_CLIENT_ID=01:44:44:44:44:44:44
 	 * ---end---
 	 */
 	g_object_class_install_property
@@ -713,11 +747,12 @@ nm_setting_ip4_config_class_init (NMSettingIP4ConfigClass *ip4_class)
 
 	/* ---ifcfg-rh---
 	 * property: dad-timeout
-	 * variable: ARPING_WAIT
-	 * default: missing variable means global default (config override or 3)
-	 * description: Timeout (in seconds) for performing DAD before configuring
-	 * IPv4 addresses. 0 turns off the DAD completely, -1 means default value.
-	 * example: ARPING_WAIT=2
+	 * variable: ACD_TIMEOUT, ARPING_WAIT
+	 * default: missing variable means global default (config override or zero)
+	 * description: Timeout (in milliseconds for ACD_TIMEOUT or in seconds
+	 *   for ARPING_WAIT) for address conflict detection before configuring
+	 *   IPv4 addresses. 0 turns off the ACD completely, -1 means default value.
+	 * example: ACD_TIMEOUT=2000 or ARPING_WAIT=2
 	 * ---end---
 	 */
 
