@@ -30,13 +30,12 @@ struct _NMParamSpecSpecialized {
 };
 
 #include <string.h>
-#include <math.h>
 #include <netinet/in.h>
 #include <dbus/dbus-glib.h>
 
 #include "nm-dbus-glib-types.h"
 
-/***********************************************************/
+/*****************************************************************************/
 /* _gvalues_compare */
 
 static gint _gvalues_compare (const GValue *value1, const GValue *value2);
@@ -157,15 +156,19 @@ _gvalues_compare_fixed (const GValue *value1, const GValue *value2)
 	case G_TYPE_FLOAT: {
 		gfloat val1 = g_value_get_float (value1);
 		gfloat val2 = g_value_get_float (value2);
+		float diff = val1 - val2;
+
 		/* Can't use == or != here due to inexactness of FP */
-		if (fabsf (val1 - val2) > FLOAT_FACTOR)
+		if (diff > FLOAT_FACTOR || diff < -FLOAT_FACTOR)
 			ret = val1 < val2 ? -1 : val1 > val2;
 		break;
 	}
 	case G_TYPE_DOUBLE: {
 		gdouble val1 = g_value_get_double (value1);
 		gdouble val2 = g_value_get_double (value2);
-		if (fabs (val1 - val2) > FLOAT_FACTOR)
+		double diff = val1 - val2;
+
+		if (diff > FLOAT_FACTOR || diff < -FLOAT_FACTOR)
 			ret = val1 < val2 ? -1 : val1 > val2;
 		break;
 	}
@@ -561,7 +564,7 @@ _gvalues_compare (const GValue *value1, const GValue *value2)
 	return ret;
 }
 
-/***********************************************************/
+/*****************************************************************************/
 
 static void
 param_specialized_init (GParamSpec *pspec)
@@ -638,7 +641,7 @@ _nm_param_spec_specialized (const char *name,
 	return G_PARAM_SPEC (pspec);
 }
 
-/***********************************************************/
+/*****************************************************************************/
 /* Tests */
 
 #if 0
@@ -953,8 +956,6 @@ main (int argc, char *argv[])
 {
 	DBusGConnection *bus;
 
-	nm_g_type_init ();
-
 	bus = dbus_g_bus_get (DBUS_BUS_SESSION, NULL);
 
 	compare_ints ();
